package xyz.lychee.lagfixer.libs.adventure.text;

import java.util.List;
import java.util.Objects;
import xyz.lychee.lagfixer.libs.adventure.internal.Internals;
import xyz.lychee.lagfixer.libs.adventure.text.format.Style;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;

final class KeybindComponentImpl extends AbstractComponent implements KeybindComponent {
   private final String keybind;

   static KeybindComponent create(final @NotNull List<? extends ComponentLike> children, final @NotNull Style style, final @NotNull String keybind) {
      return new KeybindComponentImpl(ComponentLike.asComponents(children, IS_NOT_EMPTY), (Style)Objects.requireNonNull(style, "style"), (String)Objects.requireNonNull(keybind, "keybind"));
   }

   KeybindComponentImpl(final @NotNull List<Component> children, final @NotNull Style style, final @NotNull String keybind) {
      super(children, style);
      this.keybind = keybind;
   }

   public @NotNull String keybind() {
      return this.keybind;
   }

   public @NotNull KeybindComponent keybind(final @NotNull String keybind) {
      return (KeybindComponent)(Objects.equals(this.keybind, keybind) ? this : create(this.children, this.style, keybind));
   }

   public @NotNull KeybindComponent children(final @NotNull List<? extends ComponentLike> children) {
      return create(children, this.style, this.keybind);
   }

   public @NotNull KeybindComponent style(final @NotNull Style style) {
      return create(this.children, style, this.keybind);
   }

   public boolean equals(final @Nullable Object other) {
      if (this == other) {
         return true;
      } else if (!(other instanceof KeybindComponent)) {
         return false;
      } else if (!super.equals(other)) {
         return false;
      } else {
         KeybindComponent that = (KeybindComponent)other;
         return Objects.equals(this.keybind, that.keybind());
      }
   }

   public int hashCode() {
      int result = super.hashCode();
      result = 31 * result + this.keybind.hashCode();
      return result;
   }

   public String toString() {
      return Internals.toString(this);
   }

   @NotNull
   public KeybindComponent.@NotNull Builder toBuilder() {
      return new BuilderImpl(this);
   }

   static final class BuilderImpl extends AbstractComponentBuilder<KeybindComponent, KeybindComponent.Builder> implements KeybindComponent.Builder {
      private @Nullable String keybind;

      BuilderImpl() {
      }

      BuilderImpl(final @NotNull KeybindComponent component) {
         super(component);
         this.keybind = component.keybind();
      }

      @NotNull
      public KeybindComponent.@NotNull Builder keybind(final @NotNull String keybind) {
         this.keybind = (String)Objects.requireNonNull(keybind, "keybind");
         return this;
      }

      public @NotNull KeybindComponent build() {
         if (this.keybind == null) {
            throw new IllegalStateException("keybind must be set");
         } else {
            return KeybindComponentImpl.create(this.children, this.buildStyle(), this.keybind);
         }
      }
   }
}
